import type { Either } from '@lokalise/node-core';
import type { ZodSchema } from 'zod';
export type LogFormatter<MessagePayloadSchema> = (message: MessagePayloadSchema) => unknown;
export type BarrierResult<BarrierOutput> = BarrierResultPositive<BarrierOutput> | BarrierResultNegative;
export type BarrierResultPositive<BarrierOutput> = {
    isPassing: true;
    output: BarrierOutput;
};
export type BarrierResultNegative = {
    isPassing: false;
    output?: never;
};
export type BarrierCallbackMultiConsumers<MessagePayloadSchema extends object, ExecutionContext, BarrierOutput> = (message: MessagePayloadSchema, context: ExecutionContext) => Promise<BarrierResult<BarrierOutput>>;
export declare const defaultLogFormatter: <MessagePayloadSchema>(message: MessagePayloadSchema) => MessagePayloadSchema;
export type HandlerConfigOptions<MessagePayloadSchema extends object, ExecutionContext, BarrierOutput> = {
    messageLogFormatter?: LogFormatter<MessagePayloadSchema>;
    preHandlerBarrier?: BarrierCallbackMultiConsumers<MessagePayloadSchema, ExecutionContext, BarrierOutput>;
};
export declare class MessageHandlerConfig<const MessagePayloadSchema extends object, const ExecutionContext, const BarrierOutput = unknown> {
    readonly schema: ZodSchema<MessagePayloadSchema>;
    readonly handler: Handler<MessagePayloadSchema, ExecutionContext, BarrierOutput>;
    readonly messageLogFormatter: LogFormatter<MessagePayloadSchema>;
    readonly preHandlerBarrier?: BarrierCallbackMultiConsumers<MessagePayloadSchema, ExecutionContext, BarrierOutput>;
    constructor(schema: ZodSchema<MessagePayloadSchema>, handler: Handler<MessagePayloadSchema, ExecutionContext, BarrierOutput>, options?: HandlerConfigOptions<MessagePayloadSchema, ExecutionContext, BarrierOutput>);
}
export declare class MessageHandlerConfigBuilder<MessagePayloadSchemas extends object, ExecutionContext> {
    private readonly configs;
    constructor();
    addConfig<MessagePayloadSchema extends MessagePayloadSchemas, const BarrierOutput>(schema: ZodSchema<MessagePayloadSchema>, handler: Handler<MessagePayloadSchema, ExecutionContext, BarrierOutput>, options?: HandlerConfigOptions<MessagePayloadSchema, ExecutionContext, BarrierOutput>): this;
    build(): MessageHandlerConfig<MessagePayloadSchemas, ExecutionContext, any>[];
}
export type Handler<MessagePayloadSchemas, ExecutionContext, BarrierOutput = undefined> = (message: MessagePayloadSchemas, context: ExecutionContext, barrierOutput: BarrierOutput) => Promise<Either<'retryLater', 'success'>>;
export type HandlerContainerOptions<MessagePayloadSchemas extends object, ExecutionContext> = {
    messageHandlers: MessageHandlerConfig<MessagePayloadSchemas, ExecutionContext, unknown>[];
    messageTypeField: string;
};
export declare class HandlerContainer<MessagePayloadSchemas extends object, ExecutionContext> {
    private readonly messageHandlers;
    private readonly messageTypeField;
    constructor(options: HandlerContainerOptions<MessagePayloadSchemas, ExecutionContext>);
    resolveHandler<BarrierResult>(messageType: string): MessageHandlerConfig<MessagePayloadSchemas, ExecutionContext, BarrierResult>;
    private resolveHandlerMap;
}
